using System.Collections.ObjectModel;
using System.Text.Json;
using View_by_Distance.Shared.Models;
using View_by_Distance.Shared.Models.Properties;
using View_by_Distance.Shared.Models.Stateless;

namespace View_by_Distance.Distance.Models.Stateless;

internal static class FilterLogicA
{

    private static bool? CanReMap(LocationContainer locationContainer)
    {
        bool? result = null;
        ArgumentNullException.ThrowIfNull(locationContainer);
        // if (locationContainer.WholePercentages is null)
        //     result = null;
        // else
        //     throw new NotImplementedException();
        return result;
    }

    private static bool? IsFocusPerson(DistanceSettings distanceSettings, ReadOnlyCollections readOnlyCollections, LocationContainer locationContainer)
    {
        bool? result;
        if (distanceSettings.SkipPersonWithMoreThen < 1 && readOnlyCollections.JLinkResolvedPersonKeys.Count == 0)
            result = null;
        else if (locationContainer.WholePercentages is null)
            result = null;
        else
            throw new NotImplementedException();
        return result;
    }

    internal static ReadOnlyCollection<LocationContainer> GetPreFilterLocationContainer(DistanceSettings distanceSettings, ICompareSettings compareSettings, ICompare compare, ReadOnlyCollections readOnlyCollections, ReadOnlyDictionary<int, ReadOnlyDictionary<int, FilePath>> keyValuePairs, ReadOnlyCollection<ExifDirectory> exifDirectories)
    {
        List<LocationContainer> results = [];
        string? json;
        string? model;
        bool? canReMap;
        bool? isFocusPerson;
        bool? inSkipCollection;
        FaceEncoding? faceEncoding;
        ReadOnlyDictionary<int, FilePath>? keyValues;
        FaceRecognitionDotNet.Models.FaceEncoding? encoding;
        List<FilePathAndWholePercentages>? wholePercentagesCollection;
        ReadOnlyCollection<LocationContainer> locationContainers = FilterLogicB.GetLocationContainers(distanceSettings, compareSettings, compare, exifDirectories, nameof(FilterLogicA));
        int totalSeconds = (int)Math.Floor(new TimeSpan(DateTime.Now.Ticks - compare.Ticks).TotalSeconds);
        string message = $") PreFiltering LocationContainers Face Files Collection - {totalSeconds} total second(s)";
        compare.ConstructProgressBar(locationContainers.Count, message);
        foreach (LocationContainer locationContainer in locationContainers)
        {
            compare.Tick();
            if (locationContainer.FilePath.Id is null || locationContainer.WholePercentages is null)
                continue;
            if (keyValuePairs.TryGetValue(locationContainer.FilePath.Id.Value, out keyValues))
            {
                if (keyValues.ContainsKey(locationContainer.WholePercentages.Value))
                    continue;
            }
            if (locationContainer.ExifDirectory is null || locationContainer.FaceFile is null)
                continue;
            inSkipCollection = readOnlyCollections.SkipNotSkipCollection.TryGetValue(locationContainer.FilePath.Id.Value, out wholePercentagesCollection) && wholePercentagesCollection.Any(l => l.WholePercentages == locationContainer.WholePercentages.Value);
            if (inSkipCollection is not null && inSkipCollection.Value)
                continue;
            canReMap = CanReMap(locationContainer);
            if (canReMap is not null && !canReMap.Value)
                continue;
            isFocusPerson = IsFocusPerson(distanceSettings, readOnlyCollections, locationContainer);
            if (isFocusPerson is not null && !isFocusPerson.Value)
                continue;
            if (!string.IsNullOrEmpty(distanceSettings.FocusModel))
            {
                model = Metadata.Models.Stateless.IMetadata.GetModel(locationContainer.ExifDirectory);
                if (string.IsNullOrEmpty(model) || !model.Contains(distanceSettings.FocusModel))
                    continue;
            }
            if (!string.IsNullOrEmpty(distanceSettings.FocusDirectory))
            {
                if (!locationContainer.FilePath.DirectoryFullPath.Contains(distanceSettings.FocusDirectory))
                    continue;
            }
            json = Metadata.Models.Stateless.IMetadata.GetFaceEncoding(locationContainer.ExifDirectory);
            faceEncoding = json is null ? null : JsonSerializer.Deserialize(json, FaceEncodingGenerationContext.Default.FaceEncoding);
            if (faceEncoding is null)
                continue;
            encoding = FaceRecognitionDotNet.Models.FaceRecognition.LoadFaceEncoding(faceEncoding.RawEncoding);
            results.Add(LocationContainer.Get(locationContainer, encoding, keepExifDirectory: false));
        }
        return results.AsReadOnly();
    }

}