using Humanizer;
using System.Collections.ObjectModel;
using View_by_Distance.Metadata.Models;
using View_by_Distance.Shared.Models;
using View_by_Distance.Shared.Models.Properties;
using View_by_Distance.Shared.Models.Stateless;
using WindowsShortcutFactory;

namespace View_by_Distance.Distance.Models.Stateless;

internal static class FilterLogicD
{

    internal static string GetResizeContentDirectory(ResultSettings resultSettings, string cContentDirectory, FilePath filePath)
    {
        string result;
        CombinedEnumAndIndex cei = IPath.GetCombinedEnumAndIndex(resultSettings, filePath);
        result = Path.Combine(cContentDirectory, cei.Combined);
        return result;
    }

    internal static string GetFacePartsDirectoryX(ResultSettings resultSettings, string d2FacePartsContentDirectory, FilePath filePath)
    {
        string result;
        CombinedEnumAndIndex cei = IPath.GetCombinedEnumAndIndex(resultSettings, filePath);
        result = Path.Combine(d2FacePartsContentDirectory, cei.Combined, filePath.NameWithoutExtension);
        return result;
    }

    internal static RecordB Get(int? useFiltersCounter, bool saveIndividually, bool sortingContainersAny, string forceSingleImageHumanized, int? distancePermyriad, int? by, string? displayDirectoryName)
    {
        RecordB result;
        string byValue;
        bool isByMapping;
        bool isBySorting;
        if (by is null)
        {
            isByMapping = false;
            isBySorting = !sortingContainersAny;
            byValue = $"{nameof(IMapLogic.Mapping)}Null";
        }
        else
        {
            isByMapping = by == IMapLogic.Mapping;
            isBySorting = by == IMapLogic.Sorting;
            bool isDefaultName = displayDirectoryName is not null && IPerson.IsDefaultName(displayDirectoryName);
            if (isBySorting && displayDirectoryName is null)
                byValue = saveIndividually ? nameof(IMapLogic.Individually) : $"{nameof(IMapLogic.Sorting)} Without Person{(distancePermyriad < 2000 ? "-A" : "-Z")}";
            else if (isBySorting && useFiltersCounter.HasValue)
                byValue = $"{nameof(IMapLogic.Sorting)}{(!isDefaultName ? "-A" : "-Z")} Modified Filters - {useFiltersCounter.Value}";
            else
                byValue = IDistance.Get(saveIndividually, forceSingleImageHumanized, by.Value, isDefaultName);
        }
        result = new(byValue, isByMapping, isBySorting);
        return result;
    }

    internal static void SaveContainers(DistanceSettings distanceSettings, ICompareSettings compareSettings, ICompare compare, int? updated, ReadOnlyCollection<SaveContainer> saveContainers)
    {
        string fileName;
        string checkFile;
        string sourceFile;
        List<string> distinct = [];
        WindowsShortcut windowsShortcut;
        string[] directories = (from l in saveContainers select l.Directory).Distinct().ToArray();
        foreach (string directory in directories)
        {
            if (string.IsNullOrEmpty(directory))
                continue;
            if (!Directory.Exists(directory))
                _ = Directory.CreateDirectory(directory);
        }
        int totalSeconds = (int)Math.Floor(new TimeSpan(DateTime.Now.Ticks - compare.Ticks).TotalSeconds);
        string message;
        if (updated is null)
            message = $") {saveContainers.Count:000} save(s) - {totalSeconds} total second(s)";
        else
            message = $") {saveContainers.Count:000} save(s) - {updated} Updated - {totalSeconds} total second(s)";
        compare.ConstructProgressBar(saveContainers.Count, message);
        foreach (SaveContainer saveContainer in saveContainers)
        {
            compare.Tick();
            if (string.IsNullOrEmpty(saveContainer.Directory) || string.IsNullOrEmpty(saveContainer.CheckFile) || saveContainer.FaceFileHolder is null)
                continue;
            if (saveContainer.FacePartsFileHolder is null && saveContainer.HiddenFaceFileHolder is null && saveContainer.ResizedFileHolder is null)
            {
                checkFile = saveContainer.CheckFile;
                sourceFile = saveContainer.FaceFileHolder.FullName;
            }
            else if (!saveContainer.FaceFileHolder.Exists && saveContainer.ResizedFileHolder is not null && saveContainer.ResizedFileHolder.Exists)
            {
                checkFile = saveContainer.CheckFile;
                sourceFile = saveContainer.ResizedFileHolder.FullName;
            }
            else if (saveContainer.FaceFileHolder.Exists)
            {
                sourceFile = saveContainer.FaceFileHolder.FullName;
                checkFile = $"{saveContainer.CheckFile}{compareSettings.FacesFileNameExtension}";
            }
            else
                continue;
            if (distanceSettings.SaveIndividually)
            {
                fileName = Path.GetFileName(checkFile);
                if (distinct.Contains(fileName))
                    continue;
                distinct.Add(fileName);
            }
            if (File.Exists(checkFile))
                continue;
            File.Copy(sourceFile, checkFile);
            if (distanceSettings.SaveIndividually)
                continue;
            if (saveContainer.MakeAllHidden)
                File.SetAttributes(checkFile, FileAttributes.Hidden);
            if (saveContainer.HiddenFaceFileHolder is not null && saveContainer.HiddenFaceFileHolder.Exists)
            {
                sourceFile = saveContainer.HiddenFaceFileHolder.FullName;
                checkFile = $"{saveContainer.CheckFile}{compareSettings.FacesHiddenFileNameExtension}";
            }
            else if (saveContainer.FacePartsFileHolder is not null && saveContainer.FacePartsFileHolder.Exists)
            {
                sourceFile = saveContainer.FacePartsFileHolder.FullName;
                checkFile = $"{saveContainer.CheckFile}{compareSettings.FacesPartsFileNameExtension}";
            }
            if (File.Exists(checkFile))
                continue;
            File.Copy(sourceFile, checkFile);
            if (saveContainer.MakeAllHidden)
                File.SetAttributes(checkFile, FileAttributes.Hidden);
        }
        if (updated is null)
        {
            foreach (SaveContainer saveContainer in saveContainers)
            {
                if (string.IsNullOrEmpty(saveContainer.Directory) || string.IsNullOrEmpty(saveContainer.CheckFile) || saveContainer.ResizedFileHolder is null || !saveContainer.ResizedFileHolder.Exists)
                    continue;
                checkFile = saveContainer.CheckFile;
                sourceFile = saveContainer.ResizedFileHolder.FullName;
                if (File.Exists(checkFile))
                    continue;
                File.Copy(sourceFile, checkFile);
                if (saveContainer.MakeAllHidden)
                    File.SetAttributes(checkFile, FileAttributes.Hidden);
            }
        }
        foreach (SaveContainer saveContainer in saveContainers)
        {
            if (string.IsNullOrEmpty(saveContainer.Directory) || string.IsNullOrEmpty(saveContainer.ShortcutFile) || saveContainer.ResizedFileHolder is null || !saveContainer.ResizedFileHolder.Exists)
                continue;
            try
            {
                string description;
                if (saveContainer.FaceFileHolder is not null && saveContainer.FaceFileHolder.Name.StartsWith(saveContainer.ResizedFileHolder.Name))
                    description = saveContainer.FaceFileHolder.Name;
                else
                    description = saveContainer.ResizedFileHolder.Name;
                windowsShortcut = new() { Path = saveContainer.ResizedFileHolder.FullName, Description = description };
                windowsShortcut.Save(saveContainer.ShortcutFile);
                windowsShortcut.Dispose();
                if (saveContainer.MakeAllHidden)
                    File.SetAttributes(saveContainer.ShortcutFile, FileAttributes.Hidden);
            }
            catch (Exception)
            { }
        }
    }

    internal static ReadOnlyCollection<SaveContainer> GetSaveContainers(ResultSettings resultSettings, DistanceSettings distanceSettings, ICompareSettings compareSettings, ICompare compare, string outputResolution, ReadOnlyDictionary<string, LocationContainer> onlyOne)
    {
        List<SaveContainer> results = [];
        RecordB recordB;
        RecordC recordC;
        string segmentB;
        string checkFile;
        string? directory;
        string shortcutFile;
        string facesDirectory;
        bool isCounterPersonYear;
        string facePartsDirectory;
        FileHolder? faceFileHolder;
        SaveContainer? saveContainer;
        FileHolder? resizedFileHolder;
        int? useFiltersCounter = null;
        string resizeContentDirectory;
        FileHolder? facePartsFileHolder;
        FileHolder? hiddenFaceFileHolder;
        LocationContainer locationContainer;
        bool sortingContainersAny = onlyOne.Count > 0;
        string eResultsFullGroupDirectory = IResult.GetResultsDateGroupDirectory(resultSettings,
                                                                                 nameof(E_Distance),
                                                                                 resultSettings.ResultContent);
        string cResultsFullGroupDirectory = IResult.GetResultsFullGroupDirectory(resultSettings,
                                                                                 nameof(C_Resize),
                                                                                 outputResolution,
                                                                                 includeResizeGroup: true,
                                                                                 includeModel: false,
                                                                                 includePredictorModel: false);
        string d2ResultsFullGroupDirectory = IResult.GetResultsFullGroupDirectory(resultSettings,
                                                                                  nameof(D2_FaceParts),
                                                                                  outputResolution,
                                                                                  includeResizeGroup: true,
                                                                                  includeModel: true,
                                                                                  includePredictorModel: true);
        string cContentDirectory = Path.Combine(cResultsFullGroupDirectory, resultSettings.ResultContent);
        if (!Directory.Exists(cContentDirectory))
            _ = Directory.CreateDirectory(cContentDirectory);
        string eDistanceContentTicksDirectory = Path.Combine(eResultsFullGroupDirectory, compare.Ticks.ToString());
        if (!Directory.Exists(eDistanceContentTicksDirectory))
            _ = Directory.CreateDirectory(eDistanceContentTicksDirectory);
        string d2FacePartsContentDirectory = Path.Combine(d2ResultsFullGroupDirectory, resultSettings.ResultContent);
        if (!Directory.Exists(d2FacePartsContentDirectory))
            _ = Directory.CreateDirectory(d2FacePartsContentDirectory);
        string forceSingleImageHumanized = nameof(IMapLogic.ForceSingleImage).Humanize(LetterCasing.Title);
        string message = $") Building Save Container Collection - {(int)Math.Floor(new TimeSpan(DateTime.Now.Ticks - compare.Ticks).TotalSeconds)} total second(s)";
        compare.ConstructProgressBar(onlyOne.Count, message);
        foreach (KeyValuePair<string, LocationContainer> keyValuePair in onlyOne)
        {
            if (distanceSettings.SaveIndividually)
                break;
            locationContainer = keyValuePair.Value;
            if (locationContainer.LengthPermyriad is null || locationContainer.LengthSource is null)
                continue;
            segmentB = locationContainer.LengthPermyriad.Value.ToString().PadLeft(2, '0')[..2];
            if (locationContainer.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName is null)
                continue;
            isCounterPersonYear = IPersonBirthday.IsCounterPersonYear(locationContainer.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName.KeyTicks);
            recordB = Get(useFiltersCounter, distanceSettings.SaveIndividually, sortingContainersAny, forceSingleImageHumanized, locationContainer.LengthPermyriad, locationContainer.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName);
            recordC = Get(eDistanceContentTicksDirectory, recordB, locationContainer.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName, segmentB);
            if (string.IsNullOrEmpty(recordC.Directory) || string.IsNullOrEmpty(recordC.PersonDirectory))
                continue;
            directory = recordC.Directory;
            if (!string.IsNullOrEmpty(recordC.DebugDirectory))
                results.Add(SaveContainer.Get(recordC.DebugDirectory));
            if (locationContainer.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName is null)
            {
                if (!distanceSettings.SaveSortingWithoutPerson)
                    throw new NotSupportedException();
                if (recordC.Ticks is null)
                    continue;
            }
            results.Add(SaveContainer.Get(recordC.PersonDirectory));
            facesDirectory = locationContainer.LengthSource.DirectoryFullPath;
            faceFileHolder = FileHolder.Get(locationContainer.LengthSource.FullName);
            checkFile = Path.Combine(directory, $"{locationContainer.LengthSource.Name}");
            shortcutFile = Path.Combine(recordC.PersonDirectory, $"{locationContainer.LengthSource.Name}.lnk");
            resizeContentDirectory = GetResizeContentDirectory(resultSettings, cContentDirectory, locationContainer.LengthSource);
            facePartsDirectory = GetFacePartsDirectoryX(resultSettings, d2FacePartsContentDirectory, locationContainer.LengthSource);
            hiddenFaceFileHolder = FileHolder.Get(Path.Combine(facesDirectory, $"{locationContainer.LengthSource.NameWithoutExtension}{compareSettings.FacesHiddenFileNameExtension}"));
            facePartsFileHolder = FileHolder.Get(Path.Combine(facePartsDirectory, $"{locationContainer.LengthSource.NameWithoutExtension}{compareSettings.FacesPartsFileNameExtension}"));
            resizedFileHolder = FileHolder.Get(Path.Combine(resizeContentDirectory, $"{locationContainer.LengthSource.FileNameFirstSegment}{Path.GetExtension(locationContainer.LengthSource.NameWithoutExtension)}"));
            saveContainer = SaveContainer.Get(checkFile, directory, faceFileHolder, hiddenFaceFileHolder, facePartsFileHolder, resizedFileHolder, shortcutFile);
            results.Add(saveContainer);
        }
        return results.AsReadOnly();
    }

    private static RecordB Get(int? useFiltersCounter, bool saveIndividually, bool sortingContainersAny, string forceSingleImageHumanized, int? distancePermyriad, PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName? personKeyFormattedAndKeyTicksAndDisplayDirectoryName) =>
        Get(useFiltersCounter, saveIndividually, sortingContainersAny, forceSingleImageHumanized, distancePermyriad, personKeyFormattedAndKeyTicksAndDisplayDirectoryName is null ? null : IMapLogic.Mapping, personKeyFormattedAndKeyTicksAndDisplayDirectoryName?.DisplayDirectoryName);

    private static RecordC Get(string eDistanceContentTicksDirectory, RecordB recordB, PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName? personKeyFormattedAndKeyTicksAndDisplayDirectoryName, string segmentB)
    {
        RecordC result;
        if (string.IsNullOrEmpty(personKeyFormattedAndKeyTicksAndDisplayDirectoryName?.DisplayDirectoryName))
            throw new NotImplementedException();
        long? ticks = null;
        string? debugDirectory = Path.Combine(eDistanceContentTicksDirectory, recordB.ByValue, personKeyFormattedAndKeyTicksAndDisplayDirectoryName.KeyFormatted, personKeyFormattedAndKeyTicksAndDisplayDirectoryName.DisplayDirectoryName);
        string? directory = Path.Combine(eDistanceContentTicksDirectory, recordB.ByValue, personKeyFormattedAndKeyTicksAndDisplayDirectoryName.KeyFormatted, segmentB);
        string? personDirectory = Path.Combine(directory, personKeyFormattedAndKeyTicksAndDisplayDirectoryName.DisplayDirectoryName, "lnk");
        result = new(debugDirectory, directory, ticks, personDirectory);
        return result;
    }

    internal record RecordA(long? Ticks, string? Directory);

    internal record RecordB(string ByValue, bool IsByMapping, bool IsBySorting);

    internal record RecordC(string? DebugDirectory, string? Directory, long? Ticks, string? PersonDirectory);

}