using System.Collections.ObjectModel;
using System.Text.Json;
using System.Text.Json.Serialization;
using View_by_Distance.Shared.Models.Stateless;

namespace View_by_Distance.Shared.Models;

public record PersonContainer(int? ApproximateYears,
                              PersonBirthday[]? Birthdays,
                              ReadOnlyCollection<FilePath> DisplayDirectoryAllFilePaths,
                              string DisplayDirectoryName,
                              long? Key,
                              PersonDirectory? PersonDirectory)
{

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, PersonContainerSourceGenerationContext.Default.PersonContainer);
        return result;
    }

    public static PersonContainer Get(int? approximateYears, PersonBirthday[]? birthdays, ReadOnlyCollection<FilePath> displayDirectoryAllFilePaths, string displayDirectoryName, long? key, PersonDirectory? personDirectory) =>
        new(approximateYears, birthdays, displayDirectoryAllFilePaths, displayDirectoryName, key, personDirectory);

    public static PersonContainer Get(char[] personCharacters, PersonBirthday birthday, string displayDirectoryName, PersonDirectory personDirectory) =>
        new(IAge.GetApproximateYears(personCharacters, displayDirectoryName), [birthday], new([]), displayDirectoryName, birthday.Value.Ticks, personDirectory);

    public static PersonContainer Get(int? approximateYears, PersonBirthday birthdays, string displayDirectoryName, long key) =>
        new(approximateYears, [birthdays], new([]), displayDirectoryName, key, null);

    public static PersonContainer Get(int? approximateYears, PersonBirthday birthdays, PersonDirectory? personDirectory, string displayDirectoryName, long key) =>
        new(approximateYears, [birthdays], new([]), displayDirectoryName, key, personDirectory);

    public static PersonContainer Get(int? approximateYears, ReadOnlyCollection<FilePath> displayDirectoryAllFilePaths, string displayDirectoryName, PersonDirectory? personDirectory) =>
        new(approximateYears, null, displayDirectoryAllFilePaths, displayDirectoryName, null, personDirectory);

    public static PersonContainer Get(int? approximateYears, PersonBirthday[]? birthdays, ReadOnlyCollection<FilePath> displayDirectoryAllFilePaths, string displayDirectoryName, long? key) =>
        new(approximateYears, birthdays, displayDirectoryAllFilePaths, displayDirectoryName, key, null);

    public static bool? IsKeyIsMaxBirthday(PersonContainer personContainer) =>
        personContainer.Birthdays is null || personContainer.Key is null ? null : personContainer.Key.Value == personContainer.Birthdays.First().Value.Ticks;

    public static ReadOnlyDictionary<long, PersonContainer> Extract(ReadOnlyCollection<PersonContainer> personContainers)
    {
        Dictionary<long, PersonContainer> results = [];
        foreach (PersonContainer personContainer in personContainers)
        {
            if (personContainer.Key is null || results.ContainsKey(personContainer.Key.Value))
                continue;
            results.Add(personContainer.Key.Value, personContainer);
        }
        return results.AsReadOnly();
    }

}

[JsonSourceGenerationOptions(WriteIndented = true, DefaultIgnoreCondition = JsonIgnoreCondition.WhenWritingNull)]
[JsonSerializable(typeof(PersonContainer))]
public partial class PersonContainerSourceGenerationContext : JsonSerializerContext
{
}